//  25-09-2006
#include "spi.h"

#include "at45Const.h"
#include "at45Flash.h"




unsigned char TFlashAT45161::ReadFromBuffer(unsigned char buf_numb, unsigned short addr, unsigned short read_cnt, unsigned char *read_ptr)
/*
   AT45
*/
{
	struct TSPIData SPIDATA;
	unsigned char TxArray[5];
	if (WriteFlag) {
		if (DataFlashReady()==0) return 0;
	}

	ClearTxArray(TxArray, sizeof(TxArray));

	switch (buf_numb) {
	case 1:
		TxArray[0]=AT45_BUFFER1_READ;
		break;
	case 2:
		TxArray[0]=AT45_BUFFER2_READ;
		break;
	default:
		return 0;
	}

	TxArray[2]=addr/256;
	TxArray[3]=addr%256;
	
	SPIDATA.TxArrayPtr=TxArray;
	SPIDATA.TxCnt=5;
	SPIDATA.DataPtr=read_ptr;
	SPIDATA.DataCount=read_cnt;
	SPIDATA.ChipSelectPin=ChipSelectPin;
	SPIDATA.Operation=RECEIVE_SPI;
	SPIDATA.SetChipSelect=1;
	
	SPIOut(&SPIDATA);

	return 1;
} 
 
unsigned char TFlashAT45161::WriteToBuffer(unsigned char buf_numb, unsigned short addr, unsigned short write_cnt, unsigned char *write_ptr)
/*
  
*/
{
	struct TSPIData SPIDATA;
	unsigned char TxArray[4];
	if (WriteFlag) {
		if (DataFlashReady() == 0) return 0;
	}

	ClearTxArray(TxArray, sizeof(TxArray));

	switch (buf_numb) {
	case 1:
		TxArray[0] = AT45_BUFFER1_WRITE;
		break;
	case 2:
		TxArray[0] = AT45_BUFFER2_WRITE;
		break;
	default:
		return 0;
	}
	TxArray[2] = addr/256;
	TxArray[3] = addr%256;

	SPIDATA.TxArrayPtr=TxArray;
	SPIDATA.TxCnt=4;
	SPIDATA.DataPtr=write_ptr;
	SPIDATA.DataCount=write_cnt;
	SPIDATA.ChipSelectPin=ChipSelectPin;
	SPIDATA.Operation=SEND_SPI;
	SPIDATA.SetChipSelect=1;
	

	SPIOut(&SPIDATA);

	return 1;
} 
 
unsigned char TFlashAT45161::ReadStatus(unsigned char *read_ptr)
/*
 
*/
{
	struct TSPIData SPIDATA;
	unsigned char TxArray[1];
	ClearTxArray(TxArray, sizeof(TxArray));

	TxArray[0] = AT45_STATUS_READ;

	SPIDATA.TxArrayPtr=TxArray;
	SPIDATA.TxCnt=1;
	SPIDATA.DataPtr=read_ptr;
	SPIDATA.DataCount=1;
	SPIDATA.ChipSelectPin=ChipSelectPin;
	SPIDATA.Operation=RECEIVE_SPI;
	SPIDATA.SetChipSelect=1;


	SPIOut(&SPIDATA);
	return *(read_ptr);
}
 
unsigned char TFlashAT45161::ProgrammFlashFromBuffer(unsigned char buf_numb, unsigned short page)
/*
    flash 
*/
{
	struct TSPIData SPIDATA;
	unsigned char TxArray[4];
	if (WriteFlag) {
		if (DataFlashReady() == 0) return 0;
	}

	ClearTxArray(TxArray, sizeof(TxArray));

	switch (buf_numb) {
	case 1:
		TxArray[0] = AT45_BUFFER1_TO_FLASH;
		break;
	case 2:
		TxArray[0] = AT45_BUFFER2_TO_FLASH;
		break;
	default:
		return 0;
	}

	TxArray[1]= (page*AT_45_PAGE_SHIFT)/256;
	TxArray[2]= (page*AT_45_PAGE_SHIFT)%256;

	SPIDATA.TxArrayPtr=TxArray;
	SPIDATA.TxCnt=4;
	SPIDATA.DataPtr=TxArray;
	SPIDATA.DataCount=0;
	SPIDATA.ChipSelectPin=ChipSelectPin;
	SPIDATA.Operation=SEND_SPI;
	SPIDATA.SetChipSelect=1;



	SPIOut(&SPIDATA);
	WriteFlag = 0xFF;

	return 1;
}
 
unsigned char TFlashAT45161::CopyPageToBufferWithWaiting(unsigned char buf_numb, unsigned short page)
/*
  flash  . 
   flash
*/
{
	while (DataFlashReady()==0) ;
	
	if (CopyPageToBuffer(buf_numb, page)==0)
		return 0;

	while (DataFlashReady()==0);
	return 1;
}
 
unsigned char TFlashAT45161::CopyPageToBuffer(unsigned char buf_numb, unsigned short page)
{
	struct TSPIData SPIDATA;
	unsigned char TxArray[4];
	
	if (DataFlashReady()==0) return 0;

	ClearTxArray(TxArray, sizeof(TxArray));

	switch (buf_numb) {
	case 1:
		TxArray[0] = AT45_READFLASHPAGE_TO_BUFFER1;
		break;
	case 2:
		TxArray[0] = AT45_READFLASHPAGE_TO_BUFFER2;
		break;
	default:
		return 0;
	}
	TxArray[1] = (page*AT_45_PAGE_SHIFT)/256;
	TxArray[2] = (page*AT_45_PAGE_SHIFT)%256;

	SPIDATA.TxArrayPtr=TxArray;
	SPIDATA.TxCnt=4;
	SPIDATA.DataPtr=TxArray;
	SPIDATA.DataCount=0;
	SPIDATA.ChipSelectPin=ChipSelectPin;
	SPIDATA.Operation=SEND_SPI;
	SPIDATA.SetChipSelect=1;

	SPIOut(&SPIDATA);
	
	return 1;
}
 
unsigned char TFlashAT45161::ReadFromFlash(unsigned short page, unsigned short addr, unsigned short cnt, unsigned char *ptr)
/*
  flash
*/
{
	struct TSPIData SPIDATA;
	unsigned char TxArray[8];
	if (WriteFlag) {
		if (DataFlashReady() == 0) return 0;
	}

	ClearTxArray(TxArray, sizeof(TxArray));

	TxArray[0] = 0xE8;
	TxArray[1] = (page*AT_45_PAGE_SHIFT)/256;//(page/256)*AT_45_PAGE_SHIFT;
	TxArray[2]= ((page*AT_45_PAGE_SHIFT)%256)|(addr/256);//((page%256)*AT_45_PAGE_SHIFT)|(addr/256);
	TxArray[3] = addr%256;

	SPIDATA.TxArrayPtr=TxArray;
	SPIDATA.TxCnt=8;
	SPIDATA.DataPtr=ptr;
	SPIDATA.DataCount=cnt;
	SPIDATA.ChipSelectPin=ChipSelectPin;
	SPIDATA.Operation=RECEIVE_SPI;
	SPIDATA.SetChipSelect=1;


	SPIOut(&SPIDATA);

	return 1;
}
 
unsigned char TFlashAT45161::WritePage(unsigned char buf, unsigned short page)
/*
 
*/
{	
	if (WriteFlag) {
		if (DataFlashReady() == 0) return 0;
	}

	if (FlashDisable) return 0;

	ProgrammFlashFromBuffer(buf, page);

	return 1;
}
 
unsigned char TFlashAT45161::DataFlashReady(void)
{
	unsigned char i;

	ReadStatus(&i);

	if (i & 0x80) {
		WriteFlag = 0;
		return 1;
	}
	else {
		if (FlashDisable) return 1;
	}

	return 0;
}
 
void TFlashAT45161::ClearTxArray(unsigned char *ptr, unsigned char array_length)
{
	unsigned char i;

	for (i=0; i<array_length; i++) 
		*ptr++=0;
}
 
unsigned char TFlashAT45161::WritePageAndWaitWriteEnd(unsigned char buf, unsigned short page)
{
	while (DataFlashReady()==0);

	if (WritePage(buf, page)==0) return 0;

	while (DataFlashReady()==0);

	return 1;
}
 
unsigned char TFlashAT45161::ComparePageWithBuffer(unsigned short page, unsigned char buffer, unsigned char wait_end, unsigned char *status)
/*
     flash  -   0

*/
{
	struct TSPIData SPIDATA;
	unsigned char TxArray[4];
	ClearTxArray(TxArray, sizeof(TxArray));

	if ((buffer!=1)&&(buffer!=2)) return 0;

	if (DataFlashReady()==0) return 0;

	switch (buffer) {
	case 1:
		TxArray[0] = AT45_COMPARE_PAGE_AND_BUFFER1;
		break;
	case 2:
		TxArray[0] = AT45_COMPARE_PAGE_AND_BUFFER2;
		break;
	}
	
	TxArray[1]= (page*AT_45_PAGE_SHIFT)/256;
	TxArray[2]= (page*AT_45_PAGE_SHIFT)%256;
	SPIDATA.TxArrayPtr=TxArray;
	SPIDATA.TxCnt=4;
	SPIDATA.DataPtr=TxArray;
	SPIDATA.DataCount=0;
	SPIDATA.ChipSelectPin=ChipSelectPin;
	SPIDATA.Operation=SEND_SPI;
	SPIDATA.SetChipSelect=1;

	SPIOut(&SPIDATA);
	while (wait_end) {
		unsigned char stat;
		ReadStatus(&stat);
		if (stat&0x80) {
			wait_end=0;
			*status=stat;
		}
	}
	return 1;
}
 
